/// ==============================================================================================
/// Namespace   : ScipBe.Controls.WinForms
/// Class       : FontComboBox
/// Version     : 1.1
/// Author      : Stefan Cruysberghs
/// Website     : www.scip.be
/// Date        : September 2006 - May 2007
/// Description : A FontComboBox control which will show all installed Windows fonts 
/// and a list of all theme and recently used fonts. A preview of the font will be drawn
/// Status      : Open source - Mozilla Public License 1.1
/// ==============================================================================================

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;

namespace ScipBe.Controls.WinForms
{
  #region FontComboBox class
  /// <summary>
  /// A FontComboBox control which will show all installed Windows fonts 
  /// and a list of all theme and recently used fonts
  /// </summary>
  [ToolboxBitmap(typeof(ResourceFinder), "ScipBe.Controls.WinForms.FontComboManager.Resources.ontComboBox.ico")]
  [Category("ScipBe")]
  [Description("A FontComboBox control which will show all installed Windows fonts and a list of all theme and recently used fonts")]
  public class FontComboBox : ComboBox
  {
    public FontComboBox()
    {
      DrawMode = DrawMode.OwnerDrawVariable;
      DropDownStyle = ComboBoxStyle.DropDown;
      MaxDropDownItems = 40;
      DropDownWidth = 400;

      optionsBehavior = new FontComboOptionsBehavior();
      optionsView = new FontComboOptionsView();
    }

    // Do not serialize items because otherwise all fonts will be stored in form (new keyword)
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public new ComboBox.ObjectCollection Items
    {
      get { return base.Items; }
    }

    /// <summary>
    /// Hide DropDownStyle property because it will be fixed and set in constructor
    /// </summary>
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public new ComboBoxStyle DropDownStyle;

    /// <summary>
    /// Behavior options
    /// </summary>
    private FontComboOptionsBehavior optionsBehavior;
    [Category("Behavior")]
    [Description("Options about behavior of FontComboBox")]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
    [TypeConverter(typeof(CheckListConverter))]
    public FontComboOptionsBehavior OptionsBehavior
    {
      get { return optionsBehavior; }
      set { optionsBehavior = value; }
    }

    /// <summary>
    /// View options
    /// </summary>
    private FontComboOptionsView optionsView;
    [Category("View")]
    [Description("Options about visualization of FontComboBox")]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
    [TypeConverter(typeof(CheckListConverter))]
    public FontComboOptionsView OptionsView
    {
      get { return optionsView; }
      set { optionsView = value; }
    }

    private FontManager fontManager;
    /// <summary>
    /// Link to FontManager object which contains the lists of all available fonts
    /// </summary>
    [Category("Link")]
    [Description("Link to FontManager which contains the lists of all available fonts")]
    public FontManager FontManager
    {
      get { return fontManager; }
      set { fontManager = value; }
    }

    private FontItemDrawEngine fontItemDrawEngine;
    /// <summary>
    /// Link to FontItemDrawEngine object which will draw the items of this combobox
    /// </summary>
    [Category("Link")]
    [Description("Link to FontItemDrawEngine which will draw the items of this combobox")]
    public FontItemDrawEngine FontItemDrawEngine
    {
      get { return fontItemDrawEngine; }
      set
      {
        fontItemDrawEngine = value;
        DropDownStyle = ComboBoxStyle.Simple;
        DropDownStyle = ComboBoxStyle.DropDown;
      }
    }

    /// <summary>
    /// Populate combobox with all fonts of FontManager
    /// </summary>
    [Category("Actions")]
    [Description("Populate combobox with all fonts of FontManager")]
    public void Populate()
    {
      if (fontManager != null)
      {
        fontManager.Populate(Items, optionsView);
      }
    }

    /// <summary>
    /// OnDrawItem event which will call the linked FontItemDrawEngine
    /// </summary>
    protected override void OnDrawItem(DrawItemEventArgs e)
    {
      if (e.Index > -1)
      {
        FontItem fontItem = (Items[e.Index] as FontItem);

        if (fontItem.Family == null)
        {
          if (fontItemDrawEngine != null)
            fontItemDrawEngine.DrawCategory(this, fontItem, e);
        }
        else
        {
          if (fontItemDrawEngine != null)
            fontItemDrawEngine.DrawItem(this, fontItem, e);
        }
      }
    }

    /// <summary>
    /// OnMeasureItem event which will set the item height
    /// </summary>
    protected override void OnMeasureItem(MeasureItemEventArgs e)
    {
      if ((e.Index > -1) && (fontItemDrawEngine != null))
      {
        FontItem fontItem = (Items[e.Index] as FontItem);
        if (fontItem.Family == null)
        {
          e.ItemHeight = fontItemDrawEngine.CategoryHeight;
        }
        else
        {
          e.ItemHeight = fontItemDrawEngine.ItemHeight;
        }
      }
    }

    /// <summary>
    /// Populate or refresh items when this FontComboBox gets focused
    /// </summary>
    protected override void OnEnter(EventArgs e)
    {
      base.OnEnter(e);
      this.BeginUpdate();
      Populate();

      // Fill AutoCompleteCustomSource with all fonts of this combobox (AutoCompleteSource=CustomSource)
      // Using AutoCompleteSource=ListItems will do the same, but it creates
      // a lot of flickering when focusing the combobox
      if ((AutoCompleteCustomSource.Count == 0) && (fontManager != null))
      {
        foreach (FontItem fontItem in Items)
        {
          AutoCompleteCustomSource.Add(fontItem.ToString());
        }
      }
      this.EndUpdate();
    }

    private int oldIndex;

    /// <summary>
    /// OnDropDown event which checks if new font is selected
    /// </summary>
    protected override void OnDropDown(EventArgs e)
    {
      base.OnDropDown(e);
      oldIndex = this.SelectedIndex;
    }

    protected override void OnSelectedIndexChanged(EventArgs e)
    {
      base.OnSelectedIndexChanged(e);

      if ((SelectedIndex != oldIndex) && (optionsBehavior.AddSelectedFontToRecentlyUsed))
      {
        if (fontManager != null)
        {
          this.BeginUpdate();
          oldIndex = SelectedIndex;
          string fontName = this.Text;
          fontManager.RecentlyUsedFontItems.Add(fontName);
          Populate();
          this.Text = fontName;
          this.EndUpdate();
        }
      }
    }

    /// <summary>
    /// OnDropDownClosed event
    /// </summary>
    protected override void OnDropDownClosed(EventArgs e)
    {
      base.OnDropDownClosed(e);

      if (SelectedIndex != -1)
      {
        FontItem fontItem = (Items[SelectedIndex] as FontItem);
        if (fontItem.Family == null)
        {
          SelectedIndex = oldIndex;
        }
      }
    }
  }
  #endregion

  #region OptionsBehavior class
  /// <summary>
  /// Behavior options for FontComboBox
  /// </summary>
  public class FontComboOptionsBehavior : Object
  {
    public FontComboOptionsBehavior()
    {
      addSelectedFontToRecentlyUsed = true;
    }

    private bool addSelectedFontToRecentlyUsed;
    /// <summary>
    /// Add selected font of combobox to list of recently used fonts
    /// </summary>
    [Abbreviation("AddToUsed")]
    [Description("Add selected font of combobox to list of recently used fonts")]
    [DefaultValue(true)]
    public bool AddSelectedFontToRecentlyUsed
    {
      get { return addSelectedFontToRecentlyUsed; }
      set { addSelectedFontToRecentlyUsed = value; }
    }
  }
  #endregion

  #region OptionsView class
  /// <summary>
  /// View options for FontComboBox 
  /// </summary>
  public class FontComboOptionsView : Object
  {
    public FontComboOptionsView()
    {
      showInstalledFonts = true;
      showThemeFonts = true;
      showRecentlyUsedFonts = true;
      showSymbolFonts = true;
      showIrregularFonts = true;
    }

    /// <summary>
    /// Get IList of all visible font categories
    /// </summary>
    public IList<FontCategory> GetFontCategories()
    {
      IList<FontCategory> fontCategories = new List<FontCategory>(); 
      if (showInstalledFonts)
        fontCategories.Add(FontCategory.Installed);
      if (showThemeFonts)
        fontCategories.Add(FontCategory.Theme);
      if (showRecentlyUsedFonts)
        fontCategories.Add(FontCategory.RecentlyUsed);

      return fontCategories; 
    }

    /// <summary>
    /// Show installed (Windows) fonts
    /// </summary>
    private bool showInstalledFonts;
    [Abbreviation("Installed")]
    [Description("Show installed (Windows) fonts")]
    [DefaultValue(true)]
    public bool ShowInstalledFonts
    {
      get { return showInstalledFonts; }
      set { showInstalledFonts = value; }
    }

    /// <summary>
    /// Show theme fonts
    /// </summary>
    private bool showThemeFonts;
    [Abbreviation("Theme")]
    [Description(" Show theme fonts")]
    [DefaultValue(true)]
    public bool ShowThemeFonts
    {
      get { return showThemeFonts; }
      set { showThemeFonts = value; }
    }

    /// <summary>
    /// Show recently used fonts
    /// </summary>
    private bool showRecentlyUsedFonts;
    [Abbreviation("Used")]
    [Description("Show recently used fonts")]
    [DefaultValue(true)]
    public bool ShowRecentlyUsedFonts
    {
      get { return showRecentlyUsedFonts; }
      set { showRecentlyUsedFonts = value; }
    }

    /// <summary>
    /// Show symbol fonts in installed, theme and recently used font lists
    /// </summary>
    private bool showSymbolFonts;
    [Abbreviation("Symbol")]
    [Description("Show symbol fonts in installed, theme and recently used font lists")]
    [DefaultValue(true)]
    public bool ShowSymbolFonts
    {
        get { return showSymbolFonts; }
        set { showSymbolFonts = value; }
    }

    /// <summary>
    /// Show irregular fonts in installed, theme and recently used font lists
    /// </summary>
    private bool showIrregularFonts;
    [Abbreviation("Irregular")]
    [Description("Show irregular fonts in installed, theme and recently used font lists")]
    [DefaultValue(true)]
    public bool ShowIrregularFonts
    {
        get { return showIrregularFonts; }
        set { showIrregularFonts = value; }
    }
  }
  #endregion

  #region Design-time classes
  #region AbbreviationAttribute class
  /// <summary>
  /// Abbreviation attribute for properties
  /// </summary> 
  public class AbbreviationAttribute : Attribute
  {
    private string abbreviation;
    public AbbreviationAttribute(string abbr)
    {
      abbreviation = abbr;
    }

    public string Abbreviation
    {
      get { return abbreviation; }
    }
  }
  #endregion

  #region CheckListConverter class
  /// <summary>
  /// CheckListConverter for design-time support
  /// </summary> 
  public class CheckListConverter : ExpandableObjectConverter
  {
    public override object ConvertTo(ITypeDescriptorContext context,
     System.Globalization.CultureInfo culture,
     object value,
     Type destinationType)
    {
      string propertiesstring = "";

      PropertyInfo[] properties = (value as FontComboOptionsView).GetType().GetProperties();

      foreach (PropertyInfo property in properties)
      {
        object boolValue = property.GetValue(value, null);
        if (boolValue.Equals(true))
        {
          object[] attribbutes = property.GetCustomAttributes(false);

          foreach (object attrib in attribbutes)
          {
            if ((attrib != null) && (attrib is AbbreviationAttribute))
            {
              propertiesstring = propertiesstring + "," + (attrib as AbbreviationAttribute).Abbreviation;
            }
          }
        }
      }

      return "[" + propertiesstring.Substring(1) + "]";
    }
  }
  #endregion
  #endregion
}
