/// ==============================================================================================
/// Namespace   : ScipBe.Controls.WinForms
/// Class       : FontManager
/// Version     : 1.1
/// Author      : Stefan Cruysberghs
/// Website     : www.scip.be
/// Date        : September 2006 - May 2007
/// Description : A FontManager stores lists of installed, theme and recently used fonts
/// Status      : Open source - Mozilla Public License 1.1
/// ==============================================================================================

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Text;
using System.Threading;
using System.Windows.Forms;

internal class ResourceFinder
{
}

namespace ScipBe.Controls.WinForms
{
  #region FontCategory enum
  /// <summary>
  /// Font categories
  /// </summary>
  public enum FontCategory 
  {
    Installed,
    RecentlyUsed,
    Theme
  };
  #endregion

  #region FontItem class
  /// <summary>
  /// FontItems of FontManager
  /// </summary>
  public class FontItem : Object
  {
    #region FontTextMetrics class
    /// <summary>
    /// Extra font metrics
    /// </summary>
    public class FontTextMetrics : Object
    {
      private PanoseFontFamilyTypes panoseFamilyType;
      public PanoseFontFamilyTypes PanoseFamilyType
      {
        get { return panoseFamilyType; }
        set { panoseFamilyType = value; }
      }
    }
    #endregion

    public FontItem()
    {
      textMetrics = new FontTextMetrics();
    }

    /// <summary>
    /// FontFamily
    /// </summary>
    private FontFamily fontFamily;
    public FontFamily Family
    {
      get { return fontFamily; }
      set { fontFamily = value; }
    }

    /// <summary>
    /// Category : installed, theme, recently used
    /// </summary>
    private FontCategory fontCategory;
    public FontCategory Category
    {
      get { return fontCategory; }
      set { fontCategory = value; }
    }

    /// <summary>
    /// Description of theme element
    /// </summary>
    private string description;
    public string Description
    {
      get { return description; }
      set { description = value; }
    }

    /// <summary>
    /// Extra font metrics
    /// </summary>
    private FontTextMetrics textMetrics;
    public FontTextMetrics TextMetrics
    {
      get { return textMetrics; }
      set { textMetrics = value; }
    }

    public override string ToString()
    {
      if (fontFamily == null)
      {
        return "";
      }
      else
      {
        return fontFamily.Name;
      }
    }
  }
  #endregion

  #region FontManager class
  /// <summary>
  /// A FontManager stores lists of installed, theme and recently used fonts
  /// </summary>
  [ToolboxBitmap(typeof(ResourceFinder), "ScipBe.Controls.WinForms.FontComboManager.Resources.FontManager.ico")]
  [Category("ScipBe")]
  [DefaultProperty("InstalledFontsTitle")]
  [Description("A FontManager stores lists of installed, theme and recently used fonts")]
  public class FontManager : Component
  {
    #region FontItemComparer class
    /// <summary>
    /// Compare fonts by fontname
    /// </summary>    
    public class FontItemComparer<FontItem> : IComparer<FontItem>
    {
      public int Compare(FontItem x, FontItem y)
      {
        return x.ToString().CompareTo(y.ToString());
      }
    }
    #endregion

    #region FontItemList class
    /// <summary>
    /// Generic list to store all font items
    /// </summary>    
    public class FontItemList : List<FontItem>
    {
      private FontCategory fontCategory;
      public FontCategory Category
      {
        get { return fontCategory; }
        set { fontCategory = value; }
      }

      public bool Exists(string fontName)
      {
        try
        {
          FontFamily fontFamily = new FontFamily(fontName);

          if (fontFamily != null)
          {
            FontItem fontItem = new FontItem();

            FontItemComparer<FontItem> comparer = new FontItemComparer<FontItem>();
            int index = this.BinarySearch(fontItem, comparer);
            if (index >= 0)
              return true;
            else
              return false;
          }
          else
          {
            return false;
          }
        }
        catch
        {
          return false;
        }
      }

      /// <summary>
      /// Add font, with given fontname, to this list
      /// </summary>
      public bool Add(string fontName)
      {
        return Add(fontName, "");
      }

      /// <summary>
      /// Add font, with given fontname and description, to this list
      /// </summary>
      public bool Add(string fontName, string description)
      {
        try
        {
          FontFamily fontFamily = new FontFamily(fontName);
          return Add(fontFamily, description);
        }
        catch
        {
          return false;
        }
      }

      /// <summary>
      /// Add font, with given fontfamily, to this list
      /// </summary>
      public bool Add(FontFamily fontFamily)
      {
        return Add(fontFamily, "");
      }

      /// <summary>
      /// Add font, with given fontfamily and descri, to this list
      /// </summary>
      public bool Add(FontFamily fontFamily, string description)
      {
        if (fontFamily != null)
        {
          FontItem fontItem = new FontItem();
          fontItem.Family = fontFamily;

          FontItemComparer<FontItem> comparer = new FontItemComparer<FontItem>();
          int index = this.BinarySearch(fontItem, comparer);
          if (index < 0)
          {
            fontItem.Category = this.Category;
            fontItem.Description = description;

            Add(fontItem);
            Sort(comparer);
          }

          return true;
        }
        else
        {
          return false;
        }
      }

      /// <summary>
      /// Remove font, with given fontname, from this list
      /// </summary>
      public bool Remove(string fontName)
      {
        FontFamily fontFamily = new FontFamily(fontName);
        return Remove(fontFamily);
      }

      /// <summary>
      /// Remove font, with given fontfamily, from this list
      /// </summary>
      public bool Remove(FontFamily fontFamily)
      {
        if (fontFamily != null)
        {
          FontItem fontItem = new FontItem();
          fontItem.Family = fontFamily;

          FontItemComparer<FontItem> comparer = new FontItemComparer<FontItem>();
          int index = this.BinarySearch(fontItem, comparer);
          if (index >= 0)
          {
            Remove(this[index]);
          }

          return true;
        }
        else
        {
          return false;
        }
      }
    }
    #endregion

    /// <summary>
    /// List of recently used fonts
    /// </summary>
    private FontItemList recentlyUsedFontItems;
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Browsable(false)]
    [Category("Data")]
    [Description("List of recently used fonts")]
    public FontItemList RecentlyUsedFontItems
    {
      get { return recentlyUsedFontItems; }
      //set { recentlyUsedFontItems = value; }
    }

    /// <summary>
    /// List of theme fonts
    /// </summary>
    private FontItemList themeFontItems;
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Browsable(false)]
    [Category("Data")]
    [Description("List of theme fonts")]
    public FontItemList ThemeFontItems
    {
      get { return themeFontItems; }
      //set { themeFontItems = value; }
    }

    /// <summary>
    /// List of installed (Windows) fonts
    /// </summary>
    private FontItemList installedFontItems;
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Browsable(false)]
    [Category("Data")]
    [Description("List of installed Windows fonts")]
    public FontItemList InstalledFontItems
    {
      get { return installedFontItems; }
      //set { installedFontItems = value; }
    }

    /// <summary>
    /// Title of installed fonts category
    /// </summary>
    private string installedFontsTitle;
    [Localizable(true)]
    [Category("View")]
    [Description("Title of installed fonts category")]
    public string InstalledFontsTitle
    {
      get { return installedFontsTitle; }
      set { installedFontsTitle = value; }
    }

    /// <summary>
    /// Title of theme fonts category
    /// </summary>
    private string themeFontsTitle;
    [Localizable(true)]
    [Category("View")]
      [Description("Title of theme fonts category")]
    public string ThemeFontsTitle
    {
      get { return themeFontsTitle; }
      set { themeFontsTitle = value; }
    }

    /// <summary>
    /// Title of recently used fonts category
    /// </summary>
    private string recentlyUsedFontsTitle;
    [Localizable(true)]
    [Category("View")]
    [Description("Title of recently used fonts category")]
    public string RecentlyUsedFontsTitle
    {
      get { return recentlyUsedFontsTitle; }
      set { recentlyUsedFontsTitle = value; }
    }

    public FontManager()
    {
      recentlyUsedFontItems = new FontItemList();
      recentlyUsedFontItems.Category = FontCategory.RecentlyUsed;
      themeFontItems = new FontItemList();
      themeFontItems.Category = FontCategory.Theme;
      installedFontItems = new FontItemList();
      installedFontItems.Category = FontCategory.Installed;

      if (Thread.CurrentThread.CurrentUICulture.ThreeLetterISOLanguageName == "nld")
      {
         installedFontsTitle = "Alle lettertypes";
         themeFontsTitle = "Thema lettertypes";
         recentlyUsedFontsTitle = "Recent gebruikte lettertypes";   
      }
      else
      {
        installedFontsTitle = "All fonts";
        themeFontsTitle = "Theme fonts";
        recentlyUsedFontsTitle = "Recently used fonts";
      }
    }

    /// <summary>
    /// Check if font has to be included in the fontcombobox
    /// </summary>
    protected bool IsFontVisible(FontItem fontItem, bool showIrregularFonts, bool showSymbolFonts)
    {
      if (fontItem != null)
      {
        if ((!showIrregularFonts) && (!fontItem.Family.IsStyleAvailable(FontStyle.Regular)))
        {
          return false;
        }
        else
        {
          if ((!showSymbolFonts) && (fontItem.TextMetrics.PanoseFamilyType == PanoseFontFamilyTypes.PAN_FAMILY_PICTORIAL))
            return false;
          else
            return true;
        }
      }
      else
        return false;
    }

    /// <summary>
    /// Populate FontManager with installed (Windows) fonts
    /// </summary>
    [Category("Actions")]
    [Description("Populate FontManager with installed (Windows) fonts")]
    public void Populate()
    {
      GetInstalledFonts();
      //Thread thread = new Thread(new ThreadStart(GetInstalledFonts));
      //thread.Start();
      //while (thread.ThreadState == ThreadState.Running)
      //{
      //  Thread.Sleep(5);
      //}
      //thread.Join();
    }

    /// <summary>
    /// Populate given FontComboBox items with fonts of all categories
    /// </summary>
    [Category("Actions")]
    [Description("Populate given FontComboBox items with fonts of all categories")]
    public void Populate(ComboBox.ObjectCollection comboBoxItems)
    {
      IList<FontCategory> fontCategories = new FontCategory[] { FontCategory.Installed, FontCategory.RecentlyUsed, FontCategory.Theme };
      Populate(comboBoxItems, fontCategories);
    }

    /// <summary>
    /// Populate given FontComboBox items with fonts of all given categories
    /// </summary>
    [Category("Actions")]
    [Description("Populate given FontComboBox items with fonts of all given categories")]
    public void Populate(ComboBox.ObjectCollection comboBoxItems, IList<FontCategory> fontCategories)
    {
      FontComboOptionsView optionsView = new FontComboOptionsView();
      optionsView.ShowInstalledFonts = (fontCategories.Contains(FontCategory.Installed));
      optionsView.ShowRecentlyUsedFonts = (fontCategories.Contains(FontCategory.RecentlyUsed));
      optionsView.ShowThemeFonts = (fontCategories.Contains(FontCategory.Theme));
      optionsView.ShowSymbolFonts = true;
      optionsView.ShowIrregularFonts = true;
      Populate(comboBoxItems, optionsView);
    }

    /// <summary>
    /// Populate given FontComboBox items with fonts of all given categories and view options
    /// </summary>
    [Category("Actions")]
    [Description("Populate given FontComboBox items with fonts of all given categories and view options")]
    public void Populate(ComboBox.ObjectCollection comboBoxItems, FontComboOptionsView optionsView)
    {
      int index = 0;

      // Clear all fonts
      if (optionsView.ShowInstalledFonts)
      {
        comboBoxItems.Clear();
      }
      else 
      // Clear theme and recently used fonts. 
      // List of installed Windows fonts has not been changed
      {
        while (index < comboBoxItems.Count)
        {
          FontItem fontItem = (comboBoxItems[index] as FontItem);
          if (fontItem.Category != FontCategory.Installed)
            comboBoxItems.Remove(fontItem);
          else
            index++;
        }
      }

      index = 0;

      // Add recently used fonts
      if ((optionsView.ShowRecentlyUsedFonts) && (recentlyUsedFontItems.Count > 0))
      {
        // Add category item
        FontItem fontItemRecentlyUsed = new FontItem();
        fontItemRecentlyUsed.Category = FontCategory.RecentlyUsed;
        fontItemRecentlyUsed.Description = recentlyUsedFontsTitle;
        comboBoxItems.Insert(index++, fontItemRecentlyUsed);
        // Add font items
        foreach (FontItem fontItem in recentlyUsedFontItems)
        {
          if (IsFontVisible(fontItem, optionsView.ShowIrregularFonts, optionsView.ShowSymbolFonts))
          {
            FontItem fontItemCombo = new FontItem();
            fontItemCombo.Family = fontItem.Family;
            fontItemCombo.Category = FontCategory.RecentlyUsed;
            fontItemCombo.Description = "";
            comboBoxItems.Insert(index++, fontItemCombo);
          }
        }
      }

      // Add theme fonts
      if ((optionsView.ShowThemeFonts) && (themeFontItems.Count > 0))
      {
        // Add category item
        FontItem fontItemTheme = new FontItem();
        fontItemTheme.Category = FontCategory.Theme;
        fontItemTheme.Description = themeFontsTitle;
        comboBoxItems.Insert(index++, fontItemTheme);
        // Add font items
        foreach (FontItem fontItem in themeFontItems)
        {
          if (IsFontVisible(fontItem, optionsView.ShowIrregularFonts, optionsView.ShowSymbolFonts))
          {
            FontItem fontItemCombo = new FontItem();
            fontItemCombo.Family = fontItem.Family;
            fontItemCombo.Category = FontCategory.Theme;
            fontItemCombo.Description = fontItem.Description;
            comboBoxItems.Insert(index++, fontItemCombo);
          }
        }
      }

      // Add installed fonts
      if (optionsView.ShowInstalledFonts)
      {
        // Populate installed fonts, if list is not filled yet
        if (installedFontItems.Count == 0)
        {
          GetInstalledFonts();
        }

        // Add category item when theme or recently used fonts are visible
        if (comboBoxItems.Count > 0)
        {
          FontItem fontItemInstalled = new FontItem();
          fontItemInstalled.Category = FontCategory.Installed;
          fontItemInstalled.Description = installedFontsTitle;
          comboBoxItems.Add(fontItemInstalled);
        }
        // Add font items
        foreach (FontItem fontItem in installedFontItems)
        {
          if (IsFontVisible(fontItem, optionsView.ShowIrregularFonts, optionsView.ShowSymbolFonts))
          {
            FontItem fontItemCombo = new FontItem();
            fontItemCombo.Family = fontItem.Family;
            fontItemCombo.Category = FontCategory.Installed;
            fontItemCombo.Description = "";
            comboBoxItems.Add(fontItemCombo);
          }
        }
      }
    }

    /// <summary>
    /// Get list of all installed Windows fonts and retrieve extra font metrics
    /// </summary>
    protected void GetInstalledFonts()
    {
      InstalledFontCollection installedFonts = new InstalledFontCollection();

      Graphics g = Graphics.FromHwnd(IntPtr.Zero); 
      Win32Helper win32Helper = new Win32Helper(g);

      foreach (FontFamily fontFamily in installedFonts.Families)
      {
        FontItem fontItem = new FontItem();
        fontItem.Family = fontFamily;
        fontItem.Category = FontCategory.Installed;

        Font fontFont;
        if (fontItem.Family.IsStyleAvailable(FontStyle.Regular))
        {
          fontFont = new Font(fontItem.Family, 14);
        }
        else if (fontItem.Family.IsStyleAvailable(FontStyle.Italic))
        {
          fontFont = new Font(fontItem.Family, 14, FontStyle.Italic);
        }
        else if (fontItem.Family.IsStyleAvailable(FontStyle.Bold))
        {
          fontFont = new Font(fontItem.Family, 14, FontStyle.Bold);
        }
        else
        {
          fontFont = null;
        }

        if (fontFont != null)
        {
          PanoseFontFamilyTypes panose = win32Helper.GetPanoseFontFamilyType(fontFont);

          fontItem.TextMetrics.PanoseFamilyType = panose;

          installedFontItems.Add(fontItem);
        }
      }
    }
  }
  #endregion
}

